import pandas as pd
import numpy as np
from typing import List, Optional, Tuple, Union

from pydantic import BaseModel, ValidationError

from bikeshare_model.config.core import config
from bikeshare_model.processing.data_manager import pre_pipeline_preparation



def validate_input_data(data):
    # Check for missing values
    if (data==''):
        raise ValueError("Input data contains missing values.")

    
    return True


    

def validate_inputs(*, input_df: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[dict]]:
    """Check model inputs for unprocessable values."""

    pre_processed = pre_pipeline_preparation(data_frame=input_df)
    validated_data = pre_processed[config.model_config.features].copy()
    errors = None

    try:
        # replace numpy nans so that pydantic can validate
        MultipleDataInputs(
            inputs=validated_data.replace({np.nan: None}).to_dict(orient="records")
        )
    except ValidationError as error:
        errors = error.json()

    return validated_data, errors


class DataInputSchema(BaseModel):  
    season:Optional[str]
    hr:Optional[str]
    holiday:Optional[str]
  
  

class MultipleDataInputs(BaseModel):
    inputs: List[DataInputSchema]